@@ -16,20 +16,18 @@ from sscutils.invoke_commands import (
     serialize_datascript_metadata,
     set_dvc_remotes,
     update_data,
+    validate,
     write_envs,
 )
 from sscutils.naming import (
+    DATASET_METADATA_PATHS,
     ENV_CREATION_MODULE_NAME,
     SRC_PATH,
     ProjectConfigPaths,
 )
 from sscutils.tests.create_dogshow import csv_path, package_root
 from sscutils.utils import cd_into, reset_src_module
-from sscutils.validation_functions import (
-    sql_validation,
-    validate_dataset_setup,
-    validate_project_env,
-)
+from sscutils.validation_functions import sql_validation, validate_dataset
 
 from .init_dogshow import setup_dogshow
 
@@ -42,25 +40,9 @@ def test_full_dogshow(tmp_path: Path, pytestconfig):
     pg_host = os.environ.get("POSTGRES_HOST", "localhost")
     constr = f"postgresql://postgres:postgres@{pg_host}:5432/postgres"
 
-    env_fun_script = SRC_PATH / (ENV_CREATION_MODULE_NAME + ".py")
     c = Context()
     for ds in [ds_cc.dataset_a, ds_cc.dataset_b]:
-        with ds as validator:
-            import_namespaces(c, git_commit=True)
-            lint(c)
-            with pytest.raises(DatasetSetupException):
-                validate_dataset_setup()
-            serialize_datascript_metadata(c, git_commit=True)
-            update_data(c, (csv_path,))
-            set_dvc_remotes(c)
-            write_envs(c)
-            push_envs(c, git_push=True)
-            validator()
-            validate_dataset_setup()
-            sql_validation(constr)
-            with _move_file(c, env_fun_script):
-                with pytest.raises(DatasetSetupException):
-                    validate_dataset_setup()
+        run_ds_test(ds, c)
 
     with ds_cc.project_a as validator:
         c.run(f"pip install {package_root}")
@@ -72,13 +54,11 @@ def test_full_dogshow(tmp_path: Path, pytestconfig):
             invtask = step.get_invoke_task()
             invtask(c, stage=True)
         serialize_datascript_metadata(c, git_commit=True)
-        c.run("git add reports metadata")
-        c.run('git commit -m "ran steps"')
-        c.run("git push")
-        c.run("dvc push")
+        c.run('git add reports metadata;git commit -m "ran steps"')
+        c.run("git push; dvc push")
         step.run()  # make sure it does not mess up
         validator()
-        validate_project_env()
+        validate(c)
         sql_validation(constr)
 
     with ds_cc.project_b as validator:
@@ -94,7 +74,15 @@ def test_full_dogshow(tmp_path: Path, pytestconfig):
                 ArtifactContext()
 
     with cd_into(ds_cc.get_git_remote("dataset-a"), force_clone=True):
-        validate_dataset_setup()
+        c.run("dvc pull")
+        validate(c, env="top_comps")
+        _spath = DATASET_METADATA_PATHS.table_schemas
+        bad_str = _spath.read_text().replace(
+            "name: prize_pool", "name: bad_prize_pool"
+        )
+        _spath.write_text(bad_str)
+        with pytest.raises(DatasetSetupException):
+            validate(c)
 
 
 @contextmanager
@@ -104,3 +92,25 @@ def _move_file(c, file_path):
     reset_src_module()
     yield
     c.run(f"mv -f {tmp_name} {file_path}")
+
+
+def run_ds_test(ds_context, c):
+    env_fun_script = SRC_PATH / (ENV_CREATION_MODULE_NAME + ".py")
+    with ds_context as validator:
+        import_namespaces(c, git_commit=True)
+        lint(c)
+        with pytest.raises(DatasetSetupException):
+            validate_dataset()
+        serialize_datascript_metadata(c, git_commit=True)
+        update_data(c, (csv_path,))
+        set_dvc_remotes(c)
+        write_envs(c)
+        push_envs(c, git_push=True)
+        validator()
+        sql_validation(
+            "postgresql://postgres:postgres@localhost:5432/postgres"
+        )
+        validate(c)
+        with _move_file(c, env_fun_script):
+            with pytest.raises(DatasetSetupException):
+                validate_dataset()
